package org.gcube.portlets.user.geoportaldataentry.client;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.GeoNaItemRef;
import org.gcube.application.geoportalcommon.shared.ResultSetPaginatedData;
import org.gcube.application.geoportalcommon.shared.SearchingFilter;
import org.gcube.application.geoportalcommon.shared.geoportalconfig.DocumentConfigDV;
import org.gcube.application.geoportalcommon.shared.products.ConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.content.WorkspaceContentDV;
import org.gcube.application.geoportalcommon.shared.products.paths.FileSetPathsDV;
import org.gcube.portlets.user.geoportaldataentry.client.ConstantsGeoPortalDataEntryApp.RECORD_TYPE;
import org.gcube.portlets.user.geoportaldataentry.shared.CommitReport;
import org.gcube.portlets.user.geoportaldataentry.shared.GNADataEntryExtConfigProfile;
import org.gcube.portlets.user.geoportaldataentry.shared.GeoNaFormDataObject;
import org.gcube.portlets.user.geoportaldataentry.shared.GeonaISConfig;
import org.gcube.portlets.widgets.mpformbuilder.shared.GenericDatasetBean;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * The client side stub for the RPC service.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Aug 4, 2021
 */
@RemoteServiceRelativePath("geoportaldataentryservice")
public interface GeoportalDataEntryService extends RemoteService {

	/**
	 * Save geona data forms.
	 *
	 * @param listGeonaFormObjects the list geona form objects
	 * @return the commit report
	 * @throws Exception the exception
	 */
	CommitReport saveGeonaDataForms(List<GeoNaFormDataObject> listGeonaFormObjects) throws Exception;

	/**
	 * Gets the geona init config.
	 *
	 * @return the geona init config
	 */
	GeonaISConfig getGeonaInitConfig();

	/**
	 * Gets the links for.
	 *
	 * @param itemId     the item id
	 * @param recordType the record type
	 * @return the links for
	 * @throws Exception the exception
	 */
	GeoNaItemRef getLinksFor(String itemId, RECORD_TYPE recordType) throws Exception;

	/**
	 * Gets the list concessioni.
	 *
	 * @param start             the start
	 * @param limit             the limit
	 * @param filter            the filter
	 * @param reloadFromService the reload from service
	 * @return the list concessioni
	 * @throws Exception the exception
	 */
	public ResultSetPaginatedData getListConcessioni(Integer start, Integer limit, SearchingFilter filter,
			boolean reloadFromService) throws Exception;

	/**
	 * Delete record.
	 *
	 * @param itemId     the item id
	 * @param recordType the record type
	 * @return true, if successful
	 * @throws Exception the exception
	 */
	boolean deleteRecord(String itemId, RECORD_TYPE recordType) throws Exception;

	/**
	 * Gets the JSON record.
	 *
	 * @param itemId     the item id
	 * @param recordType the record type
	 * @return the JSON record
	 * @throws Exception the exception
	 */
	String getJSONRecord(String itemId, RECORD_TYPE recordType) throws Exception;

	/**
	 * Update record.
	 *
	 * @param itemId     the item id
	 * @param jsonUpdate the json update
	 * @param recordType the record type
	 * @return the concessione DV
	 * @throws Exception the exception
	 */
	ConcessioneDV updateRecord(String itemId, String jsonUpdate, RECORD_TYPE recordType) throws Exception;

	/**
	 * Read file set paths.
	 *
	 * @return the file set paths DV
	 * @throws Exception the exception
	 */
	FileSetPathsDV readFileSetPaths() throws Exception;

	/**
	 * Gets the record.
	 *
	 * @param itemId     the item id
	 * @param recordType the record type
	 * @return the record
	 * @throws Exception the exception
	 */
	ConcessioneDV getRecord(String itemId, RECORD_TYPE recordType) throws Exception;

	/**
	 * Update record.
	 *
	 * @param itemId             the item id
	 * @param recordType         the record type
	 * @param section            the section
	 * @param pathIndex          the path index
	 * @param keepCurrentContent the keep current content
	 * @param gDBean             the g D bean
	 * @return the updated JSON string representing the itemId
	 * @throws Exception the exception
	 */
	ConcessioneDV updateSectionForRecord(String itemId, String recordType, String section, int pathIndex,
			List<WorkspaceContentDV> keepCurrentContent, GenericDatasetBean gDBean) throws Exception;

	/**
	 * Read data viewer config.
	 *
	 * @return the GNA data entry ext config profile
	 * @throws Exception the exception
	 */
	GNADataEntryExtConfigProfile readDataViewerConfig() throws Exception;

	/**
	 * Gets the document configuration for id.
	 *
	 * @param id the id
	 * @return the document configuration for id
	 */
	List<DocumentConfigDV> getListDocumentConfigurationForId(String id);

}
